/*****************************************************************************
 *
 * FILE:	config.c
 * DESCRIPTION:	parse login configuration file
 * DATE:	Sat, Nov  6 2004
 * UPDATE:	Mon, Nov  8 2004
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: config.c,v 1.1 2004/11/09 14:56:39 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <sxml.h>
#include "session.h"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	parse_config(sxml_node_t *, URI *);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
int
read_config(prog, uri)
	char *	prog;	/* program name */
	URI *	uri;	/* set returned values */
{
  char *	name;	/* configuration filename */
  int		ret = -1;

  asprintf(&name, "%s.conf", prog);
  if (name != NULL) {
    FILE *	fin;

    fin = fopen(name, "r");
    if (fin != NULL) {
      sxml_node_t *	root;

      root = sxml_parse_file(fileno(fin));
      if (root != NULL) {
	ret = parse_config(root, uri);
	sxml_delete_node(root);
      }
      fclose(fin);
    }
    free((char *)name);
  }

  return ret;
}

static int
parse_config(root, uri)
	sxml_node_t *	root;
	URI *		uri;
{
  sxml_node_t *	node;
  int		ret = -1;

  node = sxml_find_element(root, "login-configuration", NULL, NULL);
  if (node != NULL) {
    sxml_node_t *	np;

    for (np = node->child; np != NULL; np = np->next) {
      const char *	content;

      if (np->type != SXML_ELEMENT) { continue; }
      if (np->value.element.name == NULL) { continue; }

      content = sxml_get_content(np->child);
      if (content != NULL && strcmp(np->value.element.name, "uri-login") == 0) {
	uri->login = strdup(content);
	ret	   = 0;
      }
      else if (content != NULL &&
	       strcmp(np->value.element.name, "uri-startup") == 0) {
	uri->startup = strdup(content);
	ret	     = 0;
      }
      else {
	/* nothing to do */
      }
    }
  }

  return ret;
}

#if	DEBUG_CONFIG
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  URI	uri;

  if (read_config("login", &uri) == 0) {
    fprintf(stderr, "login:   %s\n", uri.login);
    fprintf(stderr, "startup: %s\n", uri.startup);
    free((char *)uri.login);
    free((char *)uri.startup);
  }
  return 0;
}
#endif	/* DEBUG_CONFIG */
