/*****************************************************************************
 *
 * FILE:	catoblepas.h
 * DESCRIPTION:	Catoblepas: simple HTML 4.01 authoring library header
 * DATE:	Wed, Feb 14 2001
 * UPDATED:	Mon, Aug 20 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2001-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2001-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: catoblepas.h,v 1.6 2007/08/20 15:33:00 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_CATOBLEPAS_H
#define	_CATOBLEPAS_H

/*****************************************************************************/

/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	CATOBLEPAS_VERSION_NUMBER	0x10003010L
#define	CATOBLEPAS_VERSION		"Catoblepas/1.0.3"
#define	CATOBLEPAS_VERSION_TEXT		CATOBLEPAS_VERSION " (2007/08/20)"
#define	CATOBLEPAS_VERSION_TEXT_LONG \
	"Catoblepas HTML Library 1.0.3, Mon, Aug 20 2007"

/*****************************************************************************/

/* useful macros */
#ifndef	EOL
#define	EOL	'\0'
#endif	/* EOL */
#ifndef	CR
#define	CR	'\015'
#endif	/* CR */
#ifndef	LF
#define	LF	'\012'
#endif	/* LF */
#ifndef	CRLF
#define	CRLF	"\015\012"
#endif	/* CRLF */
#ifndef	SPC
#define	SPC	'\040'
#endif	/* SPC */
#ifndef	TAB
#define	TAB	'\011'
#endif	/* TAB */


#ifndef	_STRING_T
#define	_STRING_T
typedef char *	String;
#endif	/* _STRING_T */

#ifndef	_BOOL_T
#define	_BOOL_T
typedef enum {	/* boolean type */
  false = 0,
  true	= 1
} bool;
#endif	/* _BOOL_T */

/*****************************************************************************/

typedef char *		Length;
typedef int		Pixels;
typedef unsigned int	Number;

typedef enum {
  HTML4,
  HTML4_Strict,
  HTML4_Transitional,
  HTML4_Frameset
} dtd_t;

typedef enum {
  LINK_NONE,
  LINK_ALTERNATE,
  LINK_STYLESHEET,
  LINK_START,
  LINK_NEXT,
  LINK_PREV,
  LINK_CONTENTS,
  LINK_INDEX,
  LINK_GLOSSARY,
  LINK_COPYRIGHT,
  LINK_CHAPTER,
  LINK_SECTION,
  LINK_SUBSECTION,
  LINK_APPENDIX,
  LINK_HELP,
  LINK_BOOKMARK
} link_t;

typedef enum {
  MEDIA_NONE,
  MEDIA_ALL,
  MEDIA_AURAL,
  MEDIA_BRAILLE,
  MEDIA_EMBOSSED,
  MEDIA_HANDHELD,
  MEDIA_PRINT,
  MEDIA_PROJECTION,
  MEDIA_SCREEN,
  MEDIA_TTY,
  MEDIA_TV
} media_t;

typedef enum {
  ALIGN_NONE,
  ALIGN_LEFT,
  ALIGN_CENTER,
  ALIGN_RIGHT,
  ALIGN_JUSTIFY,
  ALIGN_TOP,
  ALIGN_MIDDLE,
  ALIGN_BOTTOM,
  ALIGN_CHAR,
  ALIGN_BASELINE
} align_t;

typedef enum {	/* for <BR> */
  CLEAR_NONE,
  CLEAR_LEFT,
  CLEAR_RIGHT,
  CLEAR_ALL
} clear_t;

typedef enum {	/* for <UL> */
  BULLET_NONE,
  BULLET_DISC,
  BULLET_CIRCLE,
  BULLET_SQUARE
} bullet_t;

typedef enum {	/* for <OL> */
  NUMBERING_NONE,
  NUMBERING_ARABIC_NUMBER,
  NUMBERING_LOWER_ALPHA,
  NUMBERING_UPPER_ALPHA,
  NUMBERING_LOWER_ROMAN,
  NUMBERING_UPPER_ROMAN
} numbering_t;

typedef enum {	/* for <LI> */
  STYLE_NONE,
  STYLE_DISC,
  STYLE_CIRCLE,
  STYLE_SQUARE,
  STYLE_ARABIC_NUMBER,
  STYLE_LOWER_ALPHA,
  STYLE_UPPER_ALPHA,
  STYLE_LOWER_ROMAN,
  STYLE_UPPER_ROMAN
} style_t;

typedef enum {	/* for <TABLE> */
  FRAME_VOID,
  FRAME_ABOVE,
  FRAME_BELOW,
  FRAME_HSIDES,
  FRAME_LHS,
  FRAME_RHS,
  FRAME_VSIDES,
  FRAME_BOX,
  FRAME_BORDER
} frame_t;

typedef enum {	/* for <TABLE> */
  RULES_NONE,
  RULES_GROUPS,
  RULES_ROWS,
  RULES_COLS,
  RULES_ALL
} rules_t;

typedef enum {	/* for <TH>,<TD> */
  SCOPE_NONE,
  SCOPE_ROW,
  SCOPE_COL,
  SCOPE_ROWGROUP,
  SCOPE_COLGROUP
} scope_t;

typedef enum {	/* for <IFRAME>, <FRAME> */
  SCROLLING_AUTO,
  SCROLLING_YES,
  SCROLLING_NO
} scrolling_t;

typedef enum {
  METHOD_GET,
  METHOD_POST
} method_t;


typedef enum {
  EVENT_ABORT,
  EVENT_BLUR,
  EVENT_CLICK,
  EVENT_CHANGE,
  EVENT_ERROR,
  EVENT_FOCUS,
  EVENT_KEYDOWN,
  EVENT_LOAD,
  EVENT_MOUSEOUT,
  EVENT_MOUSEOVER,
  EVENT_RESET,
  EVENT_RESIZE,
  EVENT_SCROLL,
  EVENT_SUBMIT,
  EVENT_UNLOAD
} event_t;

/* event handlers */
typedef struct {
  char *	onabort;
  char *	onblur;
  char *	onclick;
  char *	onchange;
  char *	onerror;
  char *	onfocus;
  char *	onkeydown;
  char *	onload;
  char *	onmouseout;
  char *	onmouseover;
  char *	onreset;
  char *	onresize;
  char *	onscroll;
  char *	onsubmit;
  char *	onunload;
} event_handler_t;


typedef struct {
  /* Document Type Definition */
  dtd_t	dtd;

  /*
   * Strict
   */
  /* HTML/HEAD */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      char *	lang;
      void	(*init)(void);
    } attr;
  } html, head;
  /* TITLE */
  void		(*title)(const char *);	/* title */
  /* META */
  struct {
    void	(*name)(const char *, const char *);	/* name, content */
		/* http-equiv, content */
    void	(*http_equiv)(const char *, const char *);
    struct {
      char *	lang;
      void	(*init)(void);
    } attr;
  } meta;
  /* LINK */
#define	LINK	link.output()
  struct {
    void	(*output)(void);
    struct {
      char *	href;
      char *	type;	/* mime-type */
      link_t	rel;
      link_t	rev;
      media_t	media;
      char *	charset;
      void	(*init)(void);
    } attr;
  } link;
  /* BASE */
  void		(*base)(const char *);	/* URI */
  /* STYLE */
  struct {
    void	(*begin)(const char *, media_t);	/* mime-type, media */
    void	(*end)(void);
  } style;
  /* BODY */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {
	char *	background;	/* URI */
	char *	bgcolor;	/* color */
	char *	text;		/* color */
	char *	link;		/* color */
	char *	vlink;		/* color */
	char *	alink;		/* color */
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
    event_handler_t	event;	/* XXX */
  } body;
  /* SPAN/H1..H6 */
#define	SPAN(x)	span.output((x))
#define	H1(x)	h1.output((x))
#define	H2(x)	h2.output((x))
#define	H3(x)	h3.output((x))
#define	H4(x)	h4.output((x))
#define	H5(x)	h5.output((x))
#define	H6(x)	h6.output((x))
  struct _headings {
    void	(*output)(const char *);	/* text */
    struct {
      struct {
	align_t	align;	/* left|center|right|justify */
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } span, h1, h2, h3, h4, h5, h6;
  /* DIV/P */
  struct _block {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {
	align_t	align;	/* left|center|right|justify */
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
    event_handler_t	event;	/* XXX */
  } div, p;
  /* ADDRESS */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      char *	id;
      char *	class;
      char *	lang;
      void	(*init)(void);
    } attr;
  } address;
  /* PRE */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } pre;
  /* BR */
#define	BR	br.output()
  struct {
    void	(*output)(void);
    struct {
      struct {
	clear_t	clear;	/* none|left|right|all */
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } br;
  /* EM/STRONG */
#define	EM(x)		em.output((x))	
#define	STRONG(x)	strong.output((x))	
  struct _modifier {
    void	(*output)(const char *);
    struct {
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } em, strong;
  /* HR */
#define	HR	hr.output()
  struct {
    void	(*output)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* left|center|right */
	bool	noshade;
	Pixels	size;
	Length	width;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } hr;
  /* IMG */
#define	IMG(a,b,c,d)	img.output((a),(b),(c),(d))
  struct {
    void	(*output)(const char *, const char *, Length, Length);
		/* src, alt, width, height */
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* top|middle|bottom|left|right */
	Length	border;
	Pixels	hspace;
	Pixels	vspace;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
    event_handler_t	event;	/* XXX */
  } img;
  /* UL */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	bullet_t	bullet;	/* disc|circle|square */
	bool		compact;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } ul;
  /* OL */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	numbering_t	numbering;	/* 1|a|A|i|I */
	int		start;
	bool		compact;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } ol;
  /* LI */
#define	LI	li.begin()
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	style_t	style;	/* disc|circle|square|1|a|A|i|I */
	int	value;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } li;
  /* TABLE */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      char *	summary;
      Length	width;
      frame_t	frame;	/* void|above|below|hsides|lhs|rhs|vsides|box|border */
      rules_t	rules;	/* none|groups|rows|cols|all */
      Pixels	border;
      Length	cellspacing;
      Length	cellpadding;
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* left|center|right */
	char *	bgcolor;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } table;
  /* CAPTION */
#define	CAPTION(x)	caption.output((x))
  struct {
    void	(*output)(const char *);	/* caption */
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* top|bottom|left|right */
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } caption;
  /* TR */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      align_t	align;	/* left|center|right|justify|char */
      align_t	valign;	/* top|middle|bottom|baseline */
      char *	Char;
      Length	charoff;
      struct {	/* using in "HTML 4.01 Transitional" */
	char *	bgcolor;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } tr;
  /* TH/TD */
  struct _cell {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      scope_t	scope;	/* row|col|rowgroup|colgroup */
      char *	abbr;
      char *	axis;
      int	rowspan;
      int	colspan;
      align_t	align;	/* left|center|right|justify|char */
      align_t	valign;	/* top|middle|bottom|baseline */
      char *	Char;
      Length	charoff;
      struct {	/* using in "HTML 4.01 Transitional" */
	bool	nowrap;
	Pixels	width;
	Pixels	height;
	char *	bgcolor;
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } th, td;
  /* THEAD/TFOOT/TBODY */
  struct _tbl {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      align_t	align;	/* left|center|right|justify|char */
      align_t	valign;	/* top|middle|bottom|baseline */
      char *	Char;
      Length	charoff;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } thead, tfoot, tbody;
  /* A */
  struct {
    void	(*href)(const char *, const char *);	/* URI, text */
    void	(*name)(const char *, const char *);	/* Name, text */
    struct {
      char *	type;	/* mime-type */
      link_t	rel;
      link_t	rev;
      char *	target;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } a;
  /* FORM */
  struct _form {
    void	(*begin)(const char *, method_t);
    void	(*end)(void);
    struct {
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } form, multipart_form;
  /* INPUT */
  struct {
    void	(*text)(const char *, const char *, Pixels, Pixels);
    void	(*password)(const char *, const char *, Pixels, Pixels);
    void	(*checkbox)(const char *, const char *, bool);
    void	(*radio)(const char *, const char *, bool);
    void	(*submit)(const char *, const char *);
    void	(*reset)(const char *, const char *);
    void	(*file)(const char *, const char *, Pixels, Pixels);
    void	(*hidden)(const char *, const char *);
    void	(*image)(const char *, const char *, const char *);
    void	(*button)(const char *, const char *);
    struct {
      bool	readonly;	/* default: false */
      bool	disabled;	/* default: false */
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* top|middle|bottom|left|right */
      } trans;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } input;
  /* SELECT */
  struct {
			/* name, size, multiple */
    void	(*begin)(const char *, Pixels, bool);
    void	(*end)(void);
    struct {
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } select;
  /* OPTION */
#define	OPTION(x,y,z)	option.output((x),(y),(z))
  struct {
			/* text, value, selected */
    void	(*output)(const char *, const char *, bool);
    struct {
      char *	label;
      bool	disabled;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } option;
  /* TEXTAREA */
  struct {
			/* name, rows, cols */
    void	(*begin)(const char *, Number, Number);
    void	(*end)(void);
    struct {
      bool	readonly;
      bool	disabled;
      /* general attributes */
      char *	id;
      char *	class;
      char *	lang;
      char *	title;
      char *	style;
      void	(*init)(void);
    } attr;
  } textarea;

  /*
   * Transitional
   */
  /* FONT */
		/* text, size, color */
  void	(*font)(const char *, const char *, const char *);
  /* CENTER */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
  } center;
  /* IFRAME */
  struct {
    void	(*begin)(const char *, Length, Length); /* src, width, height */
    void	(*end)(void);
    struct {
      scrolling_t	scrolling;	/* auto|yes|no */
      bool		frameborder;	/* 1|0 */
      Pixels		marginwidth;
      Pixels		marginheight;
      struct {
	align_t		align;	/* left|center|right|justify */
      } trans;
      void		(*init)(void);
    } attr;
  } iframe;

  /*
   * Frameset
   */
  /* FRAMESET */
  struct {
    void	(*begin)(Length, Length);
    void	(*end)(void);
  } frameset;

  /* FRAME */
#define	FRAME(x)	frame.output((x))
  struct {
    void	(*output)(const char *);	/* src */
    struct {
      bool		noresize;
      scrolling_t	scrolling;	/* auto|yes|no */
      bool		frameborder;	/* 1|0 */
      Pixels		marginwidth;
      Pixels		marginheight;
      void		(*init)(void);
    } attr;
  } frame;

  /* NOFRAMES */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
  } noframes;

  /* print procedures */
  void	(*print)(const char *);
  void	(*printe)(const char *);
  int	(*printf)(const char *, ...);

  /* import procedures */
  int	(*template)(const char *, char **, size_t);

  void	(*done)(void);
} HTML;

/*****************************************************************************/

extern HTML *	newHTML401(dtd_t);

#endif	/* _CATOBLEPAS_H */
